-- Our key bindings.
local key = {
	left = "left",
	right = "right",
	jump = "up"
}

-- The movement values
local move = {
	maxX = 20,			-- The maximum X velocity
	maxY = 35,			-- The maximum Y velocity
	friction = 70,		-- X velocity to lose every second
	gravity = 120,		-- Y velocity to gain when not on the ground
	walking = 150,		-- X velocity to gain while walking
	jumpStart = 20,		-- Y velocity to set when jumping starts
	jumpGain = 150, 	-- Y velocity to gain while jumping
	jumpTimeMax = 0.2,  -- Seconds to remain in the jumping state
	speed = 20,			-- Speed of the entire system
}

-- Makes one value approach another value by a set amount without going over.
function approach(val1, val2, amnt)
	if math.abs(val1-val2) < amnt then return val2 end
	local sign = val1 < val2 and 1 or -1
	return val1 + sign*amnt
end


local Player = {
	x = 400,			-- X position
	y = 500,			-- Y position
	velocityX = 0,		-- X velocity
	velocityY = 0,		-- Y velocity
	jumping = false,	-- If true the player is holding down space and gaining Y velocity
	jumpTime = 0,		-- The current amount of time the player has been in the jumping state
	onGround = false,	-- If true the player is on the ground
	facing = "right",	-- The direction the player is facing
	image = love.graphics.newImage("Player.png"), -- Player image
}

-- Time for love.update. We'll be using a timestep to keep things consistant even when the game
-- slows down or speeds up
local accum = 0         -- This accumulates the dt
local step = 1/100      -- When accum is over this amount we will update everything
local updates = 0       -- The number of updates we have done in a love.update call
local maxUpdates = 50  -- The maximum number of updates in one love.update call

function love.update(dt)
    -- Increase the accumulator by the time that has passed
	accum = accum + dt
	-- If the accumulator has accrued enough time then do a logic loop
	while accum >= step do

		-- Friction and gravity
		Player.velocityX = approach(Player.velocityX, 0 , move.friction*step)
		if not Player.onGround then Player.velocityY = approach(Player.velocityY, move.maxY, move.gravity*step) end
	
		-- Movement controls
		if love.keyboard.isDown(key.right) then
			Player.velocityX = approach(Player.velocityX, move.maxX, move.walking*step)
			Player.facing = "left"
		end

		if love.keyboard.isDown(key.left) then
			Player.velocityX = approach(Player.velocityX, -move.maxX, move.walking*step)
			Player.facing = "right"
		end
	
		-- Jump gain
		if Player.jumping then
			Player.velocityY = approach(Player.velocityY, -move.maxY, move.jumpGain * math.min(step, move.jumpTimeMax - Player.jumpTime) ) 
			Player.jumpTime = Player.jumpTime + step
			if Player.jumpTime > move.jumpTimeMax then Player.jumping = false end
		end
	
		-- Landing on the ground
		if Player.y > love.graphics.getHeight() then 
			Player.y = love.graphics.getHeight() 
			Player.velocityY = 0
			Player.jumpTime = 0
			Player.jumping = false
			Player.onGround = true
		end
	
		-- Player horizontal bounds
		if Player.x > love.graphics.getWidth() then
			Player.x = love.graphics.getWidth()
			Player.velocityX = 0
		end
		if Player.x < 0 then
			Player.x = 0
			Player.velocityX = 0
		end
	
		-- Move the player
		Player.x = Player.x + Player.velocityX * move.speed * step
		Player.y = Player.y + Player.velocityY * move.speed * step
        
        -- Decrease the accumulator
		accum = accum - step
		
		-- Count the updates.  Cap the amount of updates so we don't lock up the game if it's extremely slow.
		updates = updates+1
		if updates >= maxUpdates then break end
	end
	-- Reset the updates
	updates = 0
end

function love.keypressed(k)
	-- Player hit the jump key. If on the ground then start jumping
	if k == key.jump and Player.onGround then 
		Player.velocityY = -move.jumpStart
		Player.jumping = true
		Player.onGround = false
	end
end

function love.keyreleased(k)
	-- Jump key released
	if k == key.jump then Player.jumping = false end
end

function love.draw()
	-- draw the player
	love.graphics.draw( Player.image, Player.x, Player.y, 0,  -- image, x, y, rotation
						Player.facing == "left" and 1 or -1, 1,  -- x scale, y scale
						Player.image:getWidth()/2, Player.image:getHeight() ) -- x offset, y offset
	love.graphics.print( string.format("Position: (%d,%d)", Player.x, Player.y), 5, 5)
	love.graphics.print( string.format("Velocity: (%d,%d)", Player.velocityX, Player.velocityY), 5, 25 )
	love.graphics.print( Player.onGround and "On the ground" or "Not on the ground", 5, 45)
end
